(* Delphi-Unit -
   Initialization for GnuGetText
   =============================
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Oct. 2007
   last change: Jan. 2009
   *)

unit GnuGetTextInit;

interface

uses ComCtrls, WinApiUtils;

const
// Spracheinstellung in <PrgName>.cfg
  CfgExt = 'cfg';
  LangSekt  = 'Language';
  LangID = 'LangID';
  LangIndex : integer = 0;

// Befehlszeile
  LangOption = 'lang';      // Spracheinstellung
  AltIni = 'ini';           // Ort fr alternative Ini-Datei

type
  TLanguageId = record
    Id   : string[2];
    Name : string;
    end;

// Init muss in der Projekt-Datei vor "Application.Initialize" stehen
// und definiert den zu benutzenden  Unterordner in "AppData", sowie
// die vom Progamm untersttztenn Sprachen
// Aufruf z.B.: InitTranslation('MySoftware',['en','de']);
{
  // Subdirectory in AppData for user configuration files and supported languages
  InitTranslation('JR-WeicheWare',['de','en']);
}
procedure InitTranslation (sd : string; const Ids : array of string; const Domains : array of string); overload;
procedure InitTranslation (sd : string; const Ids : array of string); overload;

// Ermittelt die Standardpfade: Anwendungsdaten, Eigene Dateien, Programm
procedure InitPaths (var AppPath,UserPath,ProgPath : string);  overload;
procedure InitPaths (var AppPath,UserPath : string); overload;
procedure InitPaths (var AppPath : string); overload;

// Ermittelt den Namen und Version/Erstellungsdatum aus den in der Exe-Datei des Prorgamms
// gespeicherten Angaben. Wenn diese fehlen, werden die Angaben ProgName,Vers,CopRgt
// verwendet
procedure InitVersion (const ProgName,Vers,CopRgt : string;
                       VersLevel,ShowLevel : integer;
                       var ProgVersName,ProgVersDate : string);

// Mehrsprachenuntersttzung ber GnuGetText
function ReadLanguageIndex : integer;
procedure SetLanguageIndex (NewLangIndex : integer);
function GetLanguageHint : string;

procedure SetLanguage (LangId : string);
function GetLanguage (Index : integer) : string;
procedure ChangeLanguage (NewLangIndex : integer); overload;
procedure ChangeLanguage (LangId : string); overload;

procedure FixToolBar(tb: TToolBar);

var
  PrgPath,PrgName  : string;
  VersInfo         : TVersionInfo;       // Programm- und Versionsinfo
  Languages        : array of TLanguageId;

implementation

uses Windows, Forms, SysUtils, gnugettext, JrUtils, WinDesktop, ShlObj, IniFiles;

var
  CfgName,AppSubDir : string;

// GnuGetText ermittelt als System-Standard nicht die Sprache sondern die lokale
// Einstellung (GetLocaleInfo)
// Bsp: engl. System mit lokaler Einstellung "German" gibt:
//    GetLocaleInfo oder GetUserDefaultLangID   ==> de_DE ($407)
//    GetUserDefaultUILanguage                  ==> 1033 = $409 = "English (US)"
// Die nachfolgenden Routinen umgehen diesen Fehler
function GetDefaultUserLang : string;
var
  pli : integer;
begin
  pli:=GetUserDefaultUILanguage and $3FF;       // not available with Win98
  if pli=0 then pli:=GetUserDefaultLangID and $3FF;
  case pli of
  7 : Result:='de';
  9 : Result:='en';
  else Result:='';
    end;
  end;

function GetLanguage (Index : integer) : string;
begin
  if Index=0 then Result:=GetdefaultUserLang else Result:=Languages[Index].Id;
  end;

procedure InitTranslation (sd : string; const Ids : array of string; const Domains : array of string);
var
  i  : integer;
begin
  AppSubDir:=sd;
  SetLength(Languages,Length(Ids)+1);
  for i:=0 to High(Ids) do with Languages[i+1] do begin
    Id:=Ids[i]; Name:='';
    end;
  for i:=0 to High(Domains) do AddDomainForResourceString(Domains[i]);
//   DefaultInstance.DebugLogToFile('pb.log');
  UseLanguage(GetLanguage(ReadLanguageIndex));
  for i:=0 to High(Languages) do with Languages[i] do begin
    if Id='' then Name:=dgettext('units','System language');
    if Id='de' then Name:=dgettext('units','German');
    if Id='en' then Name:=dgettext('units','English');
    end;
  end;

procedure InitTranslation (sd : string; const Ids : array of string);
begin
  InitTranslation(sd,Ids,['delphi7','indy10']);
  end;

{ ------------------------------------------------------------------- }
// Standardpfade
procedure InitPaths (var AppPath,UserPath,ProgPath : string);
begin
  PrgPath:=ExtractFilePath(Application.ExeName);
  PrgName:=ExtractFileName(DelExt(Application.ExeName));
  AppPath:=GetDesktopFolder(CSIDL_APPDATA);
  if length(AppPath)>0 then begin
    AppPath:=IncludeTrailingPathDelimiter(AppPath)+AppSubDir;  // Pfad zu Anwendungsdaten
    if not ForceDirectories(AppPath) then AppPath:=PrgPath;
    end
  else AppPath:=PrgPath;
  UserPath:=GetDesktopFolder(CSIDL_PERSONAL);
  ProgPath:=GetDesktopFolder(CSIDL_PROGRAM_FILES);
  end;

procedure InitPaths (var AppPath,UserPath : string);
var
  s : string;
begin
  InitPaths (AppPath,UserPath,s);
  end;

procedure InitPaths (var AppPath : string);
var
  s,t : string;
begin
  InitPaths (AppPath,t,s);
  end;

procedure InitVersion (const ProgName,Vers,CopRgt : string;
                       VersLevel,ShowLevel : integer;
                       var ProgVersName,ProgVersDate : string);
var
  i : integer;
begin
// Versions-Info
  with VersInfo do if GetVersionInfo (Application.ExeName,VersInfo) then begin
    if length(InternalName)=0 then InternalName:=ProgName;
    Comments:=Version;
    // Versionsnr. anpassen
    for i:=3 downto VersLevel do Version:=DelExt(Version);
    for i:=3 downto ShowLevel do Comments:=DelExt(Comments);
    Comments:=dgettext('units','Vers.')+' '+Comments;
    end
  else begin
    InternalName:=ProgName; Version:=Vers; Copyright:=CopRgt;
    end;
  with VersInfo do ProgVersName:=InternalName+' ('+Comments+')';
  ProgVersDate:=DateToStr(FileDateToDateTime(FileAge(Application.ExeName)));
  end;

{ ------------------------------------------------------------------- }
// Spracheinstellung laden und speichern
function ReadLanguageIndex : integer;
var
  s : string;
  i,j : integer;
  ok : boolean;
begin
  CfgName:=GetDesktopFolder(CSIDL_APPDATA);
  if length(CfgName)>0 then CfgName:=SetDirName(CfgName)+AppSubDir  // Pfad zu Anwendungsdaten
  else CfgName:=ExtractFilePath(Application.ExeName);
  CfgName:=Erweiter(CfgName,DelExt(ExtractFileName(Application.ExeName)),CfgExt);
  ok:=false; i:=0;
  for j:=1 to ParamCount do if not ok then begin   // prfe Befehlszeile
    s:=ParamStr(j);
    if (s[1]='/') or (s[1]='-') then begin
      delete (s,1,1);
      if ReadOptionValue(s,LangOption) then begin
        for i:=0 to High(Languages) do if LowerCase(s)=Languages[i].Id then break;
        ok:=i<=High(Languages);
        end;
      end;
    end;
  if not ok and FileExists(CfgName) then begin    //aus Konfigurations-Datei
    with TIniFile.Create(CfgName) do begin
      s:=ReadString(LangSekt,LangID,'');
      for i:=0 to High(Languages) do if LowerCase(s)=Languages[i].Id then break;
      if i>High(Languages) then i:=0;
      Free;
      end;
    end;
  LangIndex:=i;
  Result:=i;
  end;

procedure SetLanguageIndex (NewLangIndex : integer);
begin
  LangIndex:=NewLangIndex;
  with TIniFile.Create(CfgName) do begin
    WriteString(LangSekt,LangID,Languages[NewLangIndex].Id);
    Free;
    end;
  end;

procedure SetLanguage (LangId : string);
var
  i : integer;
begin
  for i:=0 to High(Languages) do if LowerCase(LangId)=Languages[i].Id then break;
  if i<=High(Languages) then  SetLanguageIndex(i);
  end;

function GetLanguageHint : string;
begin
  Result:=dgettext('units','Changing the language requires a restart of the program!');
  end;

{ ------------------------------------------------------------------- }
// Spracheinstellung eines laufenden Programms ndern
procedure ChangeLanguage (NewLangIndex : integer);
var
  i : integer;
begin
  SetLanguageIndex(NewLangIndex);
  UseLanguage(GetLanguage(NewLangIndex));
  with Application do for i:=0 to ComponentCount-1 do if (Components[i] is TForm) then
    ReTranslateComponent(Components[i]);
  for i:=0 to High(Languages) do with Languages[i] do begin
    if Id='' then Name:=dgettext('units','System language');
    if Id='de' then Name:=dgettext('units','German');
    if Id='en' then Name:=dgettext('units','English');
    end;
  end;

procedure ChangeLanguage (LangId : string); overload;
var
  i : integer;
begin
  for i:=0 to High(Languages) do if LowerCase(LangId)=Languages[i].Id then break;
  if i<=High(Languages) then  ChangeLanguage(i);
  end;

{ ------------------------------------------------------------------- }
// Workaround for corrupted dropdown toolbuttons when using GnuGetText
// see: http://sourceforge.net/tracker/index.php?func=detail&aid=902470&group_id=74086&atid=539908
procedure FixToolBar(tb: TToolBar);
var
  i : integer;
begin
  with tb do for i:=0 to ButtonCount-1 do with Buttons[i] do begin
    if (Style = tbsDropDown) then begin
      Style := tbsButton;
      Style := tbsDropDown;
      end;
    end;
  end;

initialization
  AppSubDir:='';
//  DefaultInstance.DebugLogToFile('ggt.log');  // enable for debugging
  SetLength(Languages,1);
  with Languages[0] do begin
    ID:=''; Name:='System';
    end;
  // Use delphi.mo for runtime library translations, if it is there
  // by putting this line into this separate unit, we can execute it
  // before the initialization sections of the other units are executed.
  // AddDomainForResourceString('delphi7');

end.

